import Oasys.gRPC


# Metaclass for static properties and constants
class ComponentType(type):
    _consts = {'ADENS', 'ADMOF', 'AM', 'AMMG', 'AMMS', 'AREA', 'AV', 'AX', 'AY', 'AZ', 'BAED', 'BAEN', 'BBED', 'BEAM', 'BEAX', 'BENL', 'BENLD', 'BENLP', 'BEP', 'BFMV', 'BFR', 'BFX', 'BFY', 'BFZ', 'BIE', 'BIED', 'BKEN', 'BKEND', 'BKENP', 'BMXX', 'BMY1', 'BMY2', 'BMYY', 'BMZ1', 'BMZ2', 'BMZZ', 'BPE1', 'BPE2', 'BRM', 'BRXX', 'BRY1', 'BRY2', 'BRZ1', 'BRZ2', 'BSAX', 'BSEN', 'BSEND', 'BSENP', 'BSXX', 'BSXY', 'BSZX', 'BV', 'BX', 'BY', 'BZ', 'CAREA', 'CFGX', 'CFGY', 'CFGZ', 'CFLX', 'CFLY', 'CFLZ', 'CFM', 'CSN', 'CST', 'CSX', 'CSY', 'CV', 'CX', 'CY', 'CZ', 'DENS', 'DM', 'DTDT', 'DV', 'DX', 'DY', 'DZ', 'E2MAX', 'E2MIN', 'E2SHEAR', 'EAV', 'EDEN', 'EMASS', 'EMAX', 'EMID', 'EMIN', 'EMS', 'ENGMAJ', 'ENGMIN', 'ENGTHK', 'ENL', 'ENLD', 'ENLP', 'EPL', 'ERATE', 'ERATIO', 'ETEN', 'EVON', 'EXX', 'EXY', 'EYY', 'EYZ', 'EZX', 'EZZ', 'FSTRN', 'GIE', 'GKE', 'GMASS', 'GMM', 'GMX', 'GMY', 'GMZ', 'GTE', 'GVM', 'GVX', 'GVY', 'GVZ', 'HGEN', 'IN_CORE', 'KEN', 'KEND', 'KENP', 'LODE_A', 'LODE_P', 'LODE_PA', 'MADD', 'NODE', 'OTHER', 'PEAV', 'PEMAG', 'PEMAX', 'PEMID', 'PEMIN', 'PEMS', 'PETEN', 'PEXX', 'PEXY', 'PEYY', 'PEYZ', 'PEZX', 'PEZZ', 'PRAT', 'RAM', 'RAV', 'RAX', 'RAY', 'RAZ', 'RDM', 'RDV', 'RDX', 'RDY', 'RDZ', 'RENAME', 'REPLACE', 'RFX', 'RFXY', 'RFY', 'RMX', 'RMXY', 'RMY', 'RQX', 'RQY', 'RT_F', 'RT_P', 'RVM', 'RVOL', 'RVV', 'RVX', 'RVY', 'RVZ', 'S2MAX', 'S2MIN', 'S2SHEAR', 'SAV', 'SB_F', 'SB_L', 'SCALAR', 'SED', 'SEN', 'SEND', 'SENP', 'SHX', 'SMAX', 'SMID', 'SMIN', 'SMS', 'SOLID_SHELL_TSHELL', 'SOX', 'SPC_F', 'SPC_M', 'SP_E', 'SP_F', 'SP_M', 'SP_R', 'SR_P', 'STEN', 'SVON', 'SW_F', 'SW_FAIL', 'SW_S', 'SW_TIME', 'SW_TRSN', 'SXX', 'SXY', 'SYY', 'SYZ', 'SZX', 'SZZ', 'TBOT', 'TEAV', 'TEMAX', 'TEMID', 'TEMIN', 'TEMP', 'TEMS', 'TENSOR', 'TETEN', 'TEXX', 'TEXY', 'TEYY', 'TEYZ', 'TEZX', 'TEZZ', 'TFM', 'TFV', 'TFX', 'TFY', 'TFZ', 'THK', 'TMID', 'TRI', 'TSTP', 'TTOP', 'UBMS', 'UBMV', 'UNOS', 'UNOV', 'USSS', 'USST', 'VECTOR', 'VM', 'VOL', 'VV', 'VX', 'VY', 'VZ', 'XSEC_A', 'XSEC_F', 'XSEC_M', 'YMOD', 'YSTRS', 'YUTF', 'YUTP'}

    def __getattr__(cls, name):
        if name in ComponentType._consts:
            return Oasys.D3PLOT._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Component(Oasys.gRPC.OasysItem, metaclass=ComponentType):
    _props = {'component', 'dataType', 'dispose', 'location', 'name'}


    def __del__(self):
        if not Oasys.D3PLOT._connection:
            return

        Oasys.D3PLOT._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
# If one of the properties we define then get it
        if name in Component._props:
            return Oasys.D3PLOT._connection.instanceGetter(self.__class__.__name__, self._handle, name)

        raise AttributeError


    def __setattr__(self, name, value):
# If one of the properties we define then set it
        if name in Component._props:
            Oasys.D3PLOT._connection.instanceSetter(self.__class__.__name__, self._handle, name, value)
            return

# Set the property locally
        self.__dict__[name] = value


# Constructor
    def __init__(self, name, component, data, options=Oasys.gRPC.defaultArg):
        handle = Oasys.D3PLOT._connection.constructor(self.__class__.__name__, name, component, data, options)
        Oasys.gRPC.OasysItem.__init__(self, self.__class__.__name__, handle)
        """
        Creates a new user defined binary data component in D3PLOT

        Parameters
        ----------
        name : string
            Name for the component
        component : constant
            The type of component stored in the user defined binary component. Either Component.NODE,
            Component.BEAM, Component.SOLID_SHELL_TSHELL or
            Component.OTHER
        data : constant
            The type of data stored in the user defined binary component. Either Component.SCALAR,
            Component.TENSOR or Component.VECTOR
        options : dict
            Optional. Dictionary containing extra information. Can contain any of:

        Returns
        -------
        Model
            Model object
        """


# Static methods
    def First():
        """
        Returns the first user defined binary component in D3PLOT (or None if there are no components)

        Returns
        -------
        Component
            Component object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "First")

    def GetFromID(number):
        """
        Returns the user defined binary component in D3PLOT by ID (or None if the component does not exist)

        Parameters
        ----------
        number : integer
            number of the component you want the Component object for

        Returns
        -------
        Component
            Component object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFromID", number)

    def GetFromName(name):
        """
        Returns the user defined binary component in D3PLOT by name (or None if the component does not exist)

        Parameters
        ----------
        name : string
            name of the component you want the Component object for

        Returns
        -------
        Component
            Component object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFromName", name)

    def Last():
        """
        Returns the last user defined binary component in D3PLOT (or None if there are no components)

        Returns
        -------
        Component
            Component object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Last")

    def Total():
        """
        Returns the total number of user defined binary components in D3PLOT

        Returns
        -------
        integer
            Total number of user binary components
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Total")



# Instance methods
    def Delete(self):
        """
        Deletes the next user defined binary data component.
        Do not use the component object after calling this method

        Returns
        -------
        Component
            Component object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Delete")

    def GetData(self, item, options=Oasys.gRPC.defaultArg):
        """
        Returns the user defined binary data component for an item

        Parameters
        ----------
        item : Node|Beam|Shell|Solid|Tshell
            The Node, Beam, Shell, Solid or
            Tshell the data should be retrieved for
        options : dict
            Optional. Dictionary containing extra information. Can contain any of:

        Returns
        -------
        float|array
            The component data
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "GetData", item, options)

    def Next(self):
        """
        Returns the next user defined binary data component (or None if there is not one)

        Returns
        -------
        Component
            Component object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Next")

    def Previous(self):
        """
        Returns the previous user defined binary data component (or None if there is not one)

        Returns
        -------
        Component
            Component object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Previous")

    def PutData(self, item, data, options=Oasys.gRPC.defaultArg):
        """
        Sets the user defined binary data component for an item

        Parameters
        ----------
        item : Node|Beam|Shell|Solid|Tshell
            The Node, Beam, Shell, Solid or
            Tshell the data should be set for
        data : float|list
            The data to set. If the component data property is Component.SCALAR
            this will be a single value. If the component data property is Component.VECTOR
            this is a list with length 3. If the component data property is Component.TESNOR
            this is an array with length 6
        options : dict
            Optional. Dictionary containing extra information. Can contain any of:

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "PutData", item, data, options)




from .beam                           import Beam
from .colour                         import Colour
from .component                      import Component
from .constant                       import Constant
from .contact                        import Contact
from .graphicswindow                 import GraphicsWindow
from .group                          import Group
from .image                          import Image
from .include                        import Include
from .material                       import Material
from .measure                        import Measure
from .model                          import Model
from .node                           import Node
from .options                        import Options
from .page                           import Page
from .part                           import Part
from .segment                        import Segment
from .setbeam                        import SetBeam
from .setnode                        import SetNode
from .setpart                        import SetPart
from .setshell                       import SetShell
from .setsolid                       import SetSolid
from .settshell                      import SetTshell
from .shell                          import Shell
from .solid                          import Solid
from .tshell                         import Tshell
from .type                           import Type
from .utils                          import Utils
from .view                           import View
from .window                         import Window
from .workflow                       import Workflow

from .itemobject                     import ItemObject
from .funcs                          import *
from .core                           import *



import Oasys.gRPC


# Metaclass for static properties and constants
class SetNodeType(type):

    def __getattr__(cls, name):

        raise AttributeError


class SetNode(Oasys.gRPC.OasysItem, metaclass=SetNodeType):
    _props = {'include', 'index', 'label', 'model', 'title', 'total', 'type'}


    def __del__(self):
        if not Oasys.D3PLOT._connection:
            return

        Oasys.D3PLOT._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
# If one of the properties we define then get it
        if name in SetNode._props:
            return Oasys.D3PLOT._connection.instanceGetter(self.__class__.__name__, self._handle, name)

        raise AttributeError


    def __setattr__(self, name, value):
# If one of the properties we define then set it
        if name in SetNode._props:
            Oasys.D3PLOT._connection.instanceSetter(self.__class__.__name__, self._handle, name, value)
            return

# Set the property locally
        self.__dict__[name] = value


# Static methods
    def First(model):
        """
        Returns the first node set in the model (or None if there are no node sets in the model)

        Parameters
        ----------
        model : Model
            Model to get first node set in

        Returns
        -------
        SetNode
            SetNode object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "First", model)

    def FlagAll(model, flag):
        """
        Flags all of the node sets in the model with a defined flag

        Parameters
        ----------
        model : Model
            Model that all the node sets will be flagged in
        flag : Flag
            Flag (see AllocateFlag) to set on the node sets

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "FlagAll", model, flag)

    def GetAll(model):
        """
        Gets all of the node sets in the model

        Parameters
        ----------
        model : Model
            Model that all the node sets are in

        Returns
        -------
        array
            Array of :py:class:`SetNode<SetNode>` objects
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetAll", model)

    def GetFlagged(model, flag):
        """
        Gets all of the node sets in the model flagged with a defined flag

        Parameters
        ----------
        model : Model
            Model that the flagged node sets are in
        flag : Flag
            Flag (see AllocateFlag) set on the node sets to get

        Returns
        -------
        array
            Array of :py:class:`SetNode<SetNode>` objects
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFlagged", model, flag)

    def GetFromID(model, label):
        """
        Returns the SetNode object for node set in model with label (or None if it does not exist)

        Parameters
        ----------
        model : Model
            Model to get node set in
        label : integer
            The LS-DYNA label for the node set in the model

        Returns
        -------
        SetNode
            SetNode object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFromID", model, label)

    def GetFromIndex(model, index):
        """
        Returns the SetNode object for node set in model with index (or None if it does not exist)

        Parameters
        ----------
        model : Model
            Model to get node set in
        index : integer
            The D3PLOT internal index in the model for node set

        Returns
        -------
        SetNode
            SetNode object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFromIndex", model, index)

    def Last(model):
        """
        Returns the last node set in the model (or None if there are no node sets in the model)

        Parameters
        ----------
        model : Model
            Model to get last node set in

        Returns
        -------
        SetNode
            SetNode object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Last", model)

    def Total(model):
        """
        Returns the total number of node sets in the model

        Parameters
        ----------
        model : Model
            Model to get total in

        Returns
        -------
        integer
            The number of node sets
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Total", model)

    def UnflagAll(model, flag):
        """
        Unsets a defined flag on all of the node sets in the model

        Parameters
        ----------
        model : Model
            Model that the defined flag for all node sets will be unset in
        flag : Flag
            Flag (see AllocateFlag) to unset on the node sets

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "UnflagAll", model, flag)



# Instance methods
    def AllItems(self):
        """
        Returns all of the node items for the node set in the model

        Returns
        -------
        list
            list of Node objects
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "AllItems")

    def ClearFlag(self, flag):
        """
        Clears a flag on a node set

        Parameters
        ----------
        flag : Flag
            Flag (see AllocateFlag) to clear on the node set

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "ClearFlag", flag)

    def Flagged(self, flag):
        """
        Checks if the node set is flagged or not

        Parameters
        ----------
        flag : Flag
            Flag (see AllocateFlag) to test on the node set

        Returns
        -------
        boolean
            True if flagged, False if not
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Flagged", flag)

    def Item(self, index):
        """
        Returns a node item from the node set in the model

        Parameters
        ----------
        index : integer
            The index in the node set to get the node from (0 <= index < total)

        Returns
        -------
        Node
            Node object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Item", index)

    def Next(self):
        """
        Returns the next node set in the model (or None if there is not one)

        Returns
        -------
        SetNode
            SetNode object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Next")

    def Previous(self):
        """
        Returns the previous node set in the model (or None if there is not one)

        Returns
        -------
        SetNode
            SetNode object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Previous")

    def SetFlag(self, flag):
        """
        Sets a flag on a node set

        Parameters
        ----------
        flag : Flag
            Flag (see AllocateFlag) to set on the node set

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "SetFlag", flag)




import Oasys.gRPC


# Metaclass for static properties and constants
class GraphicsWindowType(type):
    _consts = {'NO_OFFSET', 'OFFSET_MODEL_SPACE', 'OFFSET_SCREEN_SPACE'}

    def __getattr__(cls, name):
        if name in GraphicsWindowType._consts:
            return Oasys.D3PLOT._connection.classGetter(cls.__name__, name)

        raise AttributeError


class GraphicsWindow(Oasys.gRPC.OasysItem, metaclass=GraphicsWindowType):
    _props = {'active', 'models', 'number', 'state', 'states'}


    def __del__(self):
        if not Oasys.D3PLOT._connection:
            return

        Oasys.D3PLOT._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
# If one of the properties we define then get it
        if name in GraphicsWindow._props:
            return Oasys.D3PLOT._connection.instanceGetter(self.__class__.__name__, self._handle, name)

        raise AttributeError


    def __setattr__(self, name, value):
# If one of the properties we define then set it
        if name in GraphicsWindow._props:
            Oasys.D3PLOT._connection.instanceSetter(self.__class__.__name__, self._handle, name, value)
            return

# Set the property locally
        self.__dict__[name] = value


# Constructor
    def __init__(self, *model):
        handle = Oasys.D3PLOT._connection.constructor(self.__class__.__name__, model)
        Oasys.gRPC.OasysItem.__init__(self, self.__class__.__name__, handle)
        """
        Creates a new graphics window in D3PLOT

        Parameters
        ----------
        *model : Model object
            The model to open in this graphics window
            This argument can be repeated if required.
            Alternatively a single array argument containing the multiple values can be given

        Returns
        -------
        GraphicsWindow
            GraphicsWindow object
        """


# Static methods
    def First():
        """
        Returns the GraphicsWindow object for the first graphics window in D3PLOT
        (or None if there are no graphics windows)

        Returns
        -------
        GraphicsWindow
            GraphicsWindow object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "First")

    def GetFromID(graphics_window_number):
        """
        Returns the GraphicsWindow object for a graphics window ID (or None if graphics window does not exist)

        Parameters
        ----------
        graphics_window_number : integer
            number of the graphics window you want the GraphicsWindow object for

        Returns
        -------
        GraphicsWindow
            GraphicsWindow object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFromID", graphics_window_number)

    def Last():
        """
        Returns the GraphicsWindow object for the last graphics window in D3PLOT
        (or None if there are no graphics windows)

        Returns
        -------
        GraphicsWindow
            GraphicsWindow object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Last")

    def Total():
        """
        Returns the total number of graphics windows in use in D3PLOT

        Returns
        -------
        integer
            Total number of graphics windows
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Total")



# Instance methods
    def AddModel(self, model):
        """
        Adds a model to a graphics window

        Parameters
        ----------
        model : Model object
            The model to add to the graphics window

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "AddModel", model)

    def Delete(self):
        """
        Deletes a graphics window in D3PLOT
        Do not use the GraphicsWindow object after calling this method

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Delete")

    def GetModelInfo(self, index):
        """
        Gets the information for a model in a graphics window

        Parameters
        ----------
        index : integer
            index of the model in the graphics window you want information for (0 <= index < models)

        Returns
        -------
        dict
            Dict with properties
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "GetModelInfo", index)

    def Next(self):
        """
        Returns the next graphics window (or None if there is not one)

        Returns
        -------
        GraphicsWindow
            GraphicsWindow object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Next")

    def Previous(self):
        """
        Returns the previous graphics window (or None if there is not one)

        Returns
        -------
        GraphicsWindow
            GraphicsWindow object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Previous")

    def Redraw(self):
        """
        Redraws the graphics window

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Redraw")

    def RemoveModel(self):
        """
        Removes a model from a graphics window

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "RemoveModel")

    def SetModelInfo(self, index, info):
        """
        Sets the information for a model in a graphics window

        Parameters
        ----------
        index : integer
            index of the model in the graphics window you want to set information for (0 <= index < models)
        info : dict
            Dictionary containing the information to set. Can be any of:

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "SetModelInfo", index, info)




import Oasys.gRPC


# Metaclass for static properties and constants
class ModelType(type):

    def __getattr__(cls, name):

        raise AttributeError


class Model(Oasys.gRPC.OasysItem, metaclass=ModelType):
    _props = {'filename', 'number', 'state', 'states', 'title'}


    def __del__(self):
        if not Oasys.D3PLOT._connection:
            return

        Oasys.D3PLOT._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
# If one of the properties we define then get it
        if name in Model._props:
            return Oasys.D3PLOT._connection.instanceGetter(self.__class__.__name__, self._handle, name)

        raise AttributeError


    def __setattr__(self, name, value):
# If one of the properties we define then set it
        if name in Model._props:
            Oasys.D3PLOT._connection.instanceSetter(self.__class__.__name__, self._handle, name, value)
            return

# Set the property locally
        self.__dict__[name] = value


# Constructor
    def __init__(self, filename):
        handle = Oasys.D3PLOT._connection.constructor(self.__class__.__name__, filename)
        Oasys.gRPC.OasysItem.__init__(self, self.__class__.__name__, handle)
        """
        Reads a file into the first free model in D3PLOT

        Parameters
        ----------
        filename : string
            Filename you want to read

        Returns
        -------
        Model
            Model object
        """


# Static methods
    def First():
        """
        Returns the Model object for the first model in D3PLOT
        (or None if there are no models)

        Returns
        -------
        Model
            Model object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "First")

    def GetFromID(model_number):
        """
        Returns the Model object for a model ID (or None if model does not exist)

        Parameters
        ----------
        model_number : integer
            number of the model you want the Model object for

        Returns
        -------
        Model
            Model object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFromID", model_number)

    def Highest():
        """
        Returns the highest model number in D3PLOT (or 0 if no models). Also see Total()

        Returns
        -------
        integer
            Highest model number
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Highest")

    def Last():
        """
        Returns the Model object for the last model in D3PLOT
        (or None if there are no models)

        Returns
        -------
        Model
            Model object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Last")

    def Read(filename):
        """
        Reads a file into D3PLOT

        Parameters
        ----------
        filename : string
            Filename you want to read

        Returns
        -------
        Model
            Model object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Read", filename)

    def Total():
        """
        Returns the total number of models in use in D3PLOT. Also see Highest()

        Returns
        -------
        integer
            Total number of models in use
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Total")



# Instance methods
    def ClearFlag(self, flag):
        """
        Clears a flag on all of the items in the model

        Parameters
        ----------
        flag : Flag
            Flag (see AllocateFlag) to clear

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "ClearFlag", flag)

    def Delete(self):
        """
        Deletes a model in D3PLOT
        Do not use the Model object after calling this method

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Delete")

    def GraphicsWindows(self):
        """
        Returns the graphics window(s) that the model exists in

        Returns
        -------
        array
            Array of :py:class:`GraphicsWindow<GraphicsWindow>` objects
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "GraphicsWindows")

    def Next(self):
        """
        Returns the next model (or None if there is not one)

        Returns
        -------
        Model
            Model object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Next")

    def Previous(self):
        """
        Returns the previous model (or None if there is not one)

        Returns
        -------
        Model
            Model object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Previous")

    def ReadPropertiesFile(self, filename, info=Oasys.gRPC.defaultArg):
        """
        Reads a properties file for the model

        Parameters
        ----------
        filename : string
            Filename for the properties file you want to read
        info : dict
            Optional. Dictionary containing the information to set. Can be any of:

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "ReadPropertiesFile", filename, info)

    def Reread(self):
        """
        Rereads the model

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Reread")

    def Rescan(self):
        """
        Rescans the model

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Rescan")

    def Time(self, state):
        """
        Returns the analysis time for a particular state in the model

        Parameters
        ----------
        state : integer
            The state you want to get the time for (0 <= state <= states)

        Returns
        -------
        float
            Analysis time
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Time", state)




import Oasys.gRPC


# Metaclass for static properties and constants
class ImageType(type):
    _consts = {'AGIF', 'ALL_STATES', 'AVI', 'BMP', 'BMP8', 'BMP8C', 'GIF', 'GLB', 'GLBU', 'JPEG', 'MP4', 'PNG', 'PNG8', 'PPM', 'SCREEN', 'X2', 'X4'}

    def __getattr__(cls, name):
        if name in ImageType._consts:
            return Oasys.D3PLOT._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Image(Oasys.gRPC.OasysItem, metaclass=ImageType):


    def __del__(self):
        if not Oasys.D3PLOT._connection:
            return

        Oasys.D3PLOT._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value


# Static methods
    def Write3D(name, options=Oasys.gRPC.defaultArg):
        """
        Writes a 3D (GLB) file

        Parameters
        ----------
        name : string
            Filename for the movie
        options : dict
            Optional. Dictionary containing options for writing movie. Can be any of:

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Write3D", name, options)

    def WriteImage(name, options=Oasys.gRPC.defaultArg):
        """
        Writes a static image file

        Parameters
        ----------
        name : string
            Filename for the image
        options : dict
            Optional. Dictionary containing options for writing image. Can be any of:

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "WriteImage", name, options)

    def WriteMovie(name, options=Oasys.gRPC.defaultArg):
        """
        Writes a movie file

        Parameters
        ----------
        name : string
            Filename for the movie
        options : dict
            Optional. Dictionary containing options for writing movie. Can be any of:

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "WriteMovie", name, options)




import Oasys.gRPC


# Metaclass for static properties and constants
class MeasureType(type):
    _consts = {'AUTOMATIC', 'DEFORM', 'GENERAL', 'HIDDEN', 'LABEL', 'MAGNITUDE', 'MODEL_SPACE', 'NODE_ANGLE', 'NODE_ORIGIN', 'NODE_TO_NODE', 'NODE_TO_PART', 'PART_TO_PART', 'SCIENTIFIC', 'SHOW_ALL', 'SHOW_CURRENT', 'SHOW_NONE', 'TRANSPARENT', 'VECTOR', 'WIREFRAME'}
    _props = {'consider_blanking', 'current', 'display', 'format', 'line', 'offsets', 'precision', 'show', 'total'}

    def __getattr__(cls, name):
        if name in MeasureType._consts:
            return Oasys.D3PLOT._connection.classGetter(cls.__name__, name)
        if name in MeasureType._props:
            return Oasys.D3PLOT._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Measure(Oasys.gRPC.OasysItem, metaclass=MeasureType):
    _props = {'name'}
    _rprops = {'index', 'type'}


    def __del__(self):
        if not Oasys.D3PLOT._connection:
            return

        Oasys.D3PLOT._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
# If one of the properties we define then get it
        if name in Measure._props:
            return Oasys.D3PLOT._connection.instanceGetter(self.__class__.__name__, self._handle, name)

# If one of the read only properties we define then get it
        if name in Measure._rprops:
            return Oasys.D3PLOT._connection.instanceGetter(self.__class__.__name__, self._handle, name)

        raise AttributeError


    def __setattr__(self, name, value):
# If one of the properties we define then set it
        if name in Measure._props:
            Oasys.D3PLOT._connection.instanceSetter(self.__class__.__name__, self._handle, name, value)
            return

# If one of the read only properties we define then error
        if name in Measure._rprops:
            raise AttributeError

# Set the property locally
        self.__dict__[name] = value


# Constructor
    def __init__(self, type, options):
        handle = Oasys.D3PLOT._connection.constructor(self.__class__.__name__, type, options)
        Oasys.gRPC.OasysItem.__init__(self, self.__class__.__name__, handle)
        """
        Create a new Measure object

        Parameters
        ----------
        type : constant
            Measure type. Can be
            Measure.NODE_TO_NODE,
            Measure.NODE_ANGLE,
            Measure.NODE_ORIGIN,
            Measure.NODE_TO_PART or
            Measure.PART_TO_PART
        options : dict
            Measure options

        Returns
        -------
        Measure
            Measure object
        """


# Static methods
    def Delete(type):
        """
        Deletes the measure with the given index

        Parameters
        ----------
        type : integer
            Index of the measure to be deleted, starting at 1

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Delete", type)

    def DeleteAll():
        """
        Deletes all measures

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "DeleteAll")

    def GetFromIndex(type):
        """
        Gets the measure object for a given index

        Parameters
        ----------
        type : integer
            Index of the measure, starting at 1

        Returns
        -------
        Measure
            Measure object for that index
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFromIndex", type)



# Instance methods
    def Data(self):
        """
        Returns an object with data for this measure

        Returns
        -------
        dict
            Dict with properties
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Data")




import Oasys.gRPC


# Metaclass for static properties and constants
class ViewType(type):
    _consts = {'CURRENT', 'HIDDEN', 'ISO', 'SHADED', 'WIRE', 'XY', 'XZ', 'YZ'}

    def __getattr__(cls, name):
        if name in ViewType._consts:
            return Oasys.D3PLOT._connection.classGetter(cls.__name__, name)

        raise AttributeError


class View(Oasys.gRPC.OasysItem, metaclass=ViewType):


    def __del__(self):
        if not Oasys.D3PLOT._connection:
            return

        Oasys.D3PLOT._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value


# Static methods
    def Ac():
        """
        Autoscales the view

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Ac")

    def Ct():
        """
        Does a contour plot

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Ct")

    def Hi():
        """
        Does a Hidden line plot

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Hi")

    def Li():
        """
        Does a line (wireframe) plot

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Li")

    def Redraw():
        """
        Redraws the plot using the current plot mode

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Redraw")

    def Sh():
        """
        Does a shaded plot

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Sh")

    def Show(view_type):
        """
        Redraws using one of the standard views

        Parameters
        ----------
        view_type : constant
            The view to show. Can be +/-View.XY,
            +/-View.YZ,
            +/-View.XZ or
            +/-View.ISO

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Show", view_type)

    def Si():
        """
        Does a shaded image contour plot

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Si")

    def Vec():
        """
        Does a vector plot

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Vec")




import Oasys.gRPC


# Metaclass for static properties and constants
class PageType(type):
    _consts = {'LAYOUT_1_1', 'LAYOUT_2_2', 'LAYOUT_3_3', 'LAYOUT_CUSTOM', 'LAYOUT_TILE_TALL', 'LAYOUT_TILE_WIDE'}

    def __getattr__(cls, name):
        if name in PageType._consts:
            return Oasys.D3PLOT._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Page(Oasys.gRPC.OasysItem, metaclass=PageType):
    _props = {'layout', 'number', 'x', 'y'}


    def __del__(self):
        if not Oasys.D3PLOT._connection:
            return

        Oasys.D3PLOT._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
# If one of the properties we define then get it
        if name in Page._props:
            return Oasys.D3PLOT._connection.instanceGetter(self.__class__.__name__, self._handle, name)

        raise AttributeError


    def __setattr__(self, name, value):
# If one of the properties we define then set it
        if name in Page._props:
            Oasys.D3PLOT._connection.instanceSetter(self.__class__.__name__, self._handle, name, value)
            return

# Set the property locally
        self.__dict__[name] = value


# Static methods
    def First():
        """
        Returns the Page object for the first page in D3PLOT

        Returns
        -------
        Page
            Page object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "First")

    def GetFromID(page_number):
        """
        Returns the Page object for a page ID

        Parameters
        ----------
        page_number : integer
            number of the page you want the Page object for

        Returns
        -------
        Page
            Page object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFromID", page_number)

    def Last():
        """
        Returns the Page object for the last page in D3PLOT

        Returns
        -------
        Page
            Page object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Last")

    def Total():
        """
        Returns the total number of pages in D3PLOT

        Returns
        -------
        integer
            Total number of pages
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Total")



# Instance methods
    def Next(self):
        """
        Returns the next page (or None if there is not one)

        Returns
        -------
        Page
            Page object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Next")

    def Previous(self):
        """
        Returns the previous page (or None if there is not one)

        Returns
        -------
        Page
            Page object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Previous")

    def Show(self):
        """
        Shows this page in D3PLOT

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Show")




import os
import subprocess
import time
import Oasys.gRPC


_connection = None
_debug      = False


def start(abspath, args=None, port=50054, memory=25, license=None, debug=False, batch=False):
    if debug:
        print("Starting {}, listening on port {}".format(abspath, port))

    environ = os.environ.copy()
    if license:
        environ[ARUP_LICENSE_PATH] = license

    pargs = [abspath, "-grpc={}".format(port)]
    if args:
        pargs.extend(args)
    if batch:
        pargs.append("-batch")

    subprocess.Popen(pargs, env=environ)

# Sleep to give time for D3PLOT to start
    time.sleep(5)

    return connect(port, memory, 'localhost', debug)


def connect(port=50052, memory=25, hostname='localhost', debug=False):
    if debug:
        print("Called connect with port {} and memory {}".format(port, memory))

    Oasys.D3PLOT._debug      = debug
    Oasys.D3PLOT._connection = Oasys.gRPC.Connection("D3PLOT", port, memory, hostname, debug)
    return Oasys.D3PLOT._connection


def disconnect(connection):
    if Oasys.D3PLOT._debug:
        print("Called disconnect")

    connection.finalise()
    Oasys.D3PLOT._connection = None


def terminate(connection):
    if Oasys.D3PLOT._debug:
        print("Called terminate")

    connection.terminate()
    Oasys.D3PLOT._connection = None


def createInstance(t, h):
# Classes in D3PLOT
    if t == "Beam":
        instance = object.__new__(Oasys.D3PLOT.Beam)
    elif t == "Colour":
        instance = object.__new__(Oasys.D3PLOT.Colour)
    elif t == "Component":
        instance = object.__new__(Oasys.D3PLOT.Component)
    elif t == "Constant":
        instance = object.__new__(Oasys.D3PLOT.Constant)
    elif t == "Contact":
        instance = object.__new__(Oasys.D3PLOT.Contact)
    elif t == "GraphicsWindow":
        instance = object.__new__(Oasys.D3PLOT.GraphicsWindow)
    elif t == "Group":
        instance = object.__new__(Oasys.D3PLOT.Group)
    elif t == "Image":
        instance = object.__new__(Oasys.D3PLOT.Image)
    elif t == "Include":
        instance = object.__new__(Oasys.D3PLOT.Include)
    elif t == "Material":
        instance = object.__new__(Oasys.D3PLOT.Material)
    elif t == "Measure":
        instance = object.__new__(Oasys.D3PLOT.Measure)
    elif t == "Model":
        instance = object.__new__(Oasys.D3PLOT.Model)
    elif t == "Node":
        instance = object.__new__(Oasys.D3PLOT.Node)
    elif t == "Options":
        instance = object.__new__(Oasys.D3PLOT.Options)
    elif t == "Page":
        instance = object.__new__(Oasys.D3PLOT.Page)
    elif t == "Part":
        instance = object.__new__(Oasys.D3PLOT.Part)
    elif t == "Segment":
        instance = object.__new__(Oasys.D3PLOT.Segment)
    elif t == "SetBeam":
        instance = object.__new__(Oasys.D3PLOT.SetBeam)
    elif t == "SetNode":
        instance = object.__new__(Oasys.D3PLOT.SetNode)
    elif t == "SetPart":
        instance = object.__new__(Oasys.D3PLOT.SetPart)
    elif t == "SetShell":
        instance = object.__new__(Oasys.D3PLOT.SetShell)
    elif t == "SetSolid":
        instance = object.__new__(Oasys.D3PLOT.SetSolid)
    elif t == "SetTshell":
        instance = object.__new__(Oasys.D3PLOT.SetTshell)
    elif t == "Shell":
        instance = object.__new__(Oasys.D3PLOT.Shell)
    elif t == "Solid":
        instance = object.__new__(Oasys.D3PLOT.Solid)
    elif t == "Tshell":
        instance = object.__new__(Oasys.D3PLOT.Tshell)
    elif t == "Type":
        instance = object.__new__(Oasys.D3PLOT.Type)
    elif t == "Utils":
        instance = object.__new__(Oasys.D3PLOT.Utils)
    elif t == "View":
        instance = object.__new__(Oasys.D3PLOT.View)
    elif t == "Window":
        instance = object.__new__(Oasys.D3PLOT.Window)
    elif t == "Workflow":
        instance = object.__new__(Oasys.D3PLOT.Workflow)

# Generic object
    elif t == "ItemObject":
        instance = object.__new__(Oasys.D3PLOT.ItemObject)

# Unsupported
    else:
        raise NotImplementedError("Instance type '{}' not implemented".format(t))

    instance.__dict__['_handle']  = h;
    instance.__dict__['_objtype'] = t;

    return instance



import Oasys.gRPC


# Metaclass for static properties and constants
class TshellType(type):

    def __getattr__(cls, name):

        raise AttributeError


class Tshell(Oasys.gRPC.OasysItem, metaclass=TshellType):
    _props = {'data', 'include', 'index', 'integrationPoints', 'label', 'material', 'model', 'onPlanIntegrationPoints', 'part', 'type'}


    def __del__(self):
        if not Oasys.D3PLOT._connection:
            return

        Oasys.D3PLOT._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
# If one of the properties we define then get it
        if name in Tshell._props:
            return Oasys.D3PLOT._connection.instanceGetter(self.__class__.__name__, self._handle, name)

        raise AttributeError


    def __setattr__(self, name, value):
# If one of the properties we define then set it
        if name in Tshell._props:
            Oasys.D3PLOT._connection.instanceSetter(self.__class__.__name__, self._handle, name, value)
            return

# Set the property locally
        self.__dict__[name] = value


# Static methods
    def BlankAll(window, model):
        """
        Blanks all of the tshells in the model

        Parameters
        ----------
        window : GraphicsWindow
            GraphicsWindow) to blank the tshells in
        model : Model
            Model that all the tshells will be blanked in

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "BlankAll", window, model)

    def BlankFlagged(window, model, flag):
        """
        Blanks all of the tshells in the model flagged with a defined flag

        Parameters
        ----------
        window : GraphicsWindow
            GraphicsWindow) to blank the tshells in
        model : Model
            Model that the flagged tshells will be blanked in
        flag : Flag
            Flag (see AllocateFlag) set on the tshells to blank

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "BlankFlagged", window, model, flag)

    def First(model):
        """
        Returns the first tshell in the model (or None if there are no tshells in the model)

        Parameters
        ----------
        model : Model
            Model to get first tshell in

        Returns
        -------
        Tshell
            Tshell object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "First", model)

    def FlagAll(model, flag):
        """
        Flags all of the tshells in the model with a defined flag

        Parameters
        ----------
        model : Model
            Model that all the tshells will be flagged in
        flag : Flag
            Flag (see AllocateFlag) to set on the tshells

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "FlagAll", model, flag)

    def GetAll(model):
        """
        Gets all of the tshells in the model

        Parameters
        ----------
        model : Model
            Model that all the tshells are in

        Returns
        -------
        array
            Array of :py:class:`Tshell<Tshell>` objects
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetAll", model)

    def GetFlagged(model, flag):
        """
        Gets all of the tshells in the model flagged with a defined flag

        Parameters
        ----------
        model : Model
            Model that the flagged tshells are in
        flag : Flag
            Flag (see AllocateFlag) set on the tshells to get

        Returns
        -------
        array
            Array of :py:class:`Tshell<Tshell>` objects
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFlagged", model, flag)

    def GetFromID(model, label):
        """
        Returns the Tshell object for tshell in model with label (or None if it does not exist)

        Parameters
        ----------
        model : Model
            Model to get tshell in
        label : integer
            The LS-DYNA label for the tshell in the model

        Returns
        -------
        Tshell
            Tshell object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFromID", model, label)

    def GetFromIndex(model, index):
        """
        Returns the Tshell object for tshell in model with index (or None if it does not exist)

        Parameters
        ----------
        model : Model
            Model to get tshell in
        index : integer
            The D3PLOT internal index in the model for tshell

        Returns
        -------
        Tshell
            Tshell object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFromIndex", model, index)

    def GetMultipleData(component, items, options=Oasys.gRPC.defaultArg):
        """
        Returns the value for a data component for multiple tshells. For each tshell a local property called data will be created
        containing a number if a scalar component, or a list if a vector or tensor component (or None if the value cannot be calculated).
        The data is also returned as an object.
        Also see GetData

        Parameters
        ----------
        component : constant
            :ref:`Component constant ` to get data for
        items : list
            List of Tshell objects to get the data for.
            All of the tshells must be in the same model
        options : dict
            Optional. Dictionary containing options for getting data. Can be any of:

        Returns
        -------
        $object
            Dict containing the data. A property is created in the object for each tshell with the label. The value of the property is a number if a scalar component or an array if a vector or tensor component (or None if the value cannot be calculated)
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetMultipleData", component, items, options)

    def Last(model):
        """
        Returns the last tshell in the model (or None if there are no tshells in the model)

        Parameters
        ----------
        model : Model
            Model to get last tshell in

        Returns
        -------
        Tshell
            Tshell object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Last", model)

    def Pick():
        """
        Allows the user to pick a tshell from the screen

        Returns
        -------
        Tshell
            Tshell object or None if cancelled
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Pick")

    def Select(flag):
        """
        Selects tshells using an object menu

        Parameters
        ----------
        flag : Flag
            Flag (see AllocateFlag) to use when selecting tshells

        Returns
        -------
        integer
            The number of tshells selected or None if menu cancelled
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Select", flag)

    def Total(model):
        """
        Returns the total number of tshells in the model

        Parameters
        ----------
        model : Model
            Model to get total in

        Returns
        -------
        integer
            The number of tshells
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Total", model)

    def TotalDeleted(model):
        """
        Returns the total number of thick shells that have been deleted in a model

        Parameters
        ----------
        model : Model
            Model to get total in

        Returns
        -------
        integer
            The number of thick shells that have been deleted
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "TotalDeleted", model)

    def UnblankAll(window, model):
        """
        Unblanks all of the tshells in the model

        Parameters
        ----------
        window : GraphicsWindow
            GraphicsWindow) to unblank the tshells in
        model : Model
            Model that all the tshells will be unblanked in

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "UnblankAll", window, model)

    def UnblankFlagged(window, model, flag):
        """
        Unblanks all of the tshells in the model flagged with a defined flag

        Parameters
        ----------
        window : GraphicsWindow
            GraphicsWindow) to unblank the tshells in
        model : Model
            Model that the flagged tshells will be unblanked in
        flag : Flag
            Flag (see AllocateFlag) set on the tshells to unblank

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "UnblankFlagged", window, model, flag)

    def UnflagAll(model, flag):
        """
        Unsets a defined flag on all of the tshells in the model

        Parameters
        ----------
        model : Model
            Model that the defined flag for all tshells will be unset in
        flag : Flag
            Flag (see AllocateFlag) to unset on the tshells

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "UnflagAll", model, flag)



# Instance methods
    def Blank(self, window):
        """
        Blanks the tshell in a graphics window

        Parameters
        ----------
        window : GraphicsWindow
            GraphicsWindow) to blank the tshell in

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Blank", window)

    def Blanked(self, window):
        """
        Checks if the tshell is blanked in a graphics window or not

        Parameters
        ----------
        window : GraphicsWindow
            GraphicsWindow) in which to check if the tshell is blanked

        Returns
        -------
        boolean
            True if blanked, False if not
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Blanked", window)

    def ClearFlag(self, flag):
        """
        Clears a flag on a tshell

        Parameters
        ----------
        flag : Flag
            Flag (see AllocateFlag) to clear on the tshell

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "ClearFlag", flag)

    def Deleted(self):
        """
        Checks if the thick shell has been deleted or not

        Returns
        -------
        boolean
            True if deleted, False if not
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Deleted")

    def Flagged(self, flag):
        """
        Checks if the tshell is flagged or not

        Parameters
        ----------
        flag : Flag
            Flag (see AllocateFlag) to test on the tshell

        Returns
        -------
        boolean
            True if flagged, False if not
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Flagged", flag)

    def GetData(self, component, options=Oasys.gRPC.defaultArg):
        """
        Returns the value for a data component.
        Also see GetMultipleData

        Parameters
        ----------
        component : constant
            :ref:`Component constant ` to get data for
        options : dict
            Optional. Dictionary containing options for getting data. Can be any of:

        Returns
        -------
        float|array
            Number if a scalar component, array if a vector or tensor component (or None if the value cannot be calculated)
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "GetData", component, options)

    def LocalAxes(self):
        """
        Returns the local axes of the element in model space, expressed as direction cosines in a 2D list.
        Beam elements must have 3 nodes to be able to return local axes

        Returns
        -------
        list
            list of lists
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "LocalAxes")

    def Next(self):
        """
        Returns the next tshell in the model (or None if there is not one)

        Returns
        -------
        Tshell
            Tshell object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Next")

    def PlasticStrain(self, options=Oasys.gRPC.defaultArg):
        """
        Returns the effective plastic strain for the thick shell (or None if the value cannot be calculated)

        Parameters
        ----------
        options : dict
            Optional. Dictionary containing options for getting data. Can be any of:

        Returns
        -------
        float
            Plastic strain
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "PlasticStrain", options)

    def Previous(self):
        """
        Returns the previous tshell in the model (or None if there is not one)

        Returns
        -------
        Tshell
            Tshell object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Previous")

    def SetFlag(self, flag):
        """
        Sets a flag on a tshell

        Parameters
        ----------
        flag : Flag
            Flag (see AllocateFlag) to set on the tshell

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "SetFlag", flag)

    def StrainTensor(self, options=Oasys.gRPC.defaultArg):
        """
        Returns the strain tensor for the thick shell

        Parameters
        ----------
        options : dict
            Optional. Dictionary containing options for getting data. Can be any of:

        Returns
        -------
        array
            Array containing the strain tensor [Exx, Eyy, Ezz, Exy, Eyz, Ezx] (or None if the value cannot be calculated)
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "StrainTensor", options)

    def StressTensor(self, options=Oasys.gRPC.defaultArg):
        """
        Returns the stress tensor for the thick shell

        Parameters
        ----------
        options : dict
            Optional. Dictionary containing options for getting data. Can be any of:

        Returns
        -------
        array
            Array containing the stress tensor [Exx, Eyy, Ezz, Exy, Eyz, Ezx] (or None if the value cannot be calculated)
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "StressTensor", options)

    def Topology(self):
        """
        Returns the topology for the tshell in the model

        Returns
        -------
        list
            list of Node objects
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Topology")

    def Unblank(self, window):
        """
        Unblanks the tshell in a graphics window

        Parameters
        ----------
        window : GraphicsWindow
            GraphicsWindow) to unblank the tshell in

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Unblank", window)

    def VonMisesStress(self, options=Oasys.gRPC.defaultArg):
        """
        Returns the von Mises stress for the thick shell (or None if the value cannot be calculated)

        Parameters
        ----------
        options : dict
            Optional. Dictionary containing options for getting data. Can be any of:

        Returns
        -------
        float
            von Mises stress
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "VonMisesStress", options)




import Oasys.gRPC


# Metaclass for static properties and constants
class ConstantType(type):
    _consts = {'ALL', 'BASIC', 'BOTTOM', 'CONST_X', 'CONST_Y', 'CONST_Z', 'CURRENT_VAL', 'CUT_SECTION', 'CYLINDRICAL', 'DEFORMED', 'DELETE', 'FAMILY', 'GLOBAL', 'GT', 'GTEQ', 'INCLUDE', 'INTERSECTION', 'LEAVE', 'LOCAL', 'LS_DYNA', 'LT', 'LTEQ', 'MAGNITUDE', 'MATERIAL', 'MAX', 'MIDDLE', 'MIN', 'MODEL', 'N3', 'NEIPH', 'NEIPS', 'NEIPT', 'NIP_B', 'NIP_H', 'NIP_S', 'NIP_T', 'NORMAL', 'N_ON_PLAN', 'N_UBMS', 'N_UBMV', 'N_UNOS', 'N_UNOV', 'N_USSS', 'N_USST', 'OFF', 'OMIT', 'ON', 'OR_AND_V', 'OUTLINE', 'SCREEN', 'STATE', 'TOP', 'TRANSPARENT', 'UNION', 'USER', 'USER_DEFINED', 'X', 'XX', 'XY', 'Y', 'YY', 'YZ', 'Z', 'ZX', 'ZZ'}

    def __getattr__(cls, name):
        if name in ConstantType._consts:
            return Oasys.D3PLOT._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Constant(Oasys.gRPC.OasysItem, metaclass=ConstantType):


    def __del__(self):
        if not Oasys.D3PLOT._connection:
            return

        Oasys.D3PLOT._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value



import Oasys.gRPC


# Metaclass for static properties and constants
class GroupType(type):

    def __getattr__(cls, name):

        raise AttributeError


class Group(Oasys.gRPC.OasysItem, metaclass=GroupType):
    _props = {'label', 'model', 'title', 'type'}


    def __del__(self):
        if not Oasys.D3PLOT._connection:
            return

        Oasys.D3PLOT._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
# If one of the properties we define then get it
        if name in Group._props:
            return Oasys.D3PLOT._connection.instanceGetter(self.__class__.__name__, self._handle, name)

        raise AttributeError


    def __setattr__(self, name, value):
# If one of the properties we define then set it
        if name in Group._props:
            Oasys.D3PLOT._connection.instanceSetter(self.__class__.__name__, self._handle, name, value)
            return

# Set the property locally
        self.__dict__[name] = value


# Constructor
    def __init__(self, model):
        handle = Oasys.D3PLOT._connection.constructor(self.__class__.__name__, model)
        Oasys.gRPC.OasysItem.__init__(self, self.__class__.__name__, handle)
        """
        Creates a new group in D3PLOT

        Parameters
        ----------
        model : Model object
            The model to create the group in

        Returns
        -------
        Group
            Group object
        """


# Static methods
    def First(model):
        """
        Returns the first group in the model (or None if there are no groups)

        Parameters
        ----------
        model : Model
            Model to get first group in

        Returns
        -------
        Group
            Group object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "First", model)

    def GetFromID(model, label):
        """
        Returns the Group object for group in model with label (or None if it does not exist)

        Parameters
        ----------
        model : Model
            Model to get group in
        label : integer
            The label for the group in the model

        Returns
        -------
        Group
            Group object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFromID", model, label)

    def Last(model):
        """
        Returns the last group in the model (or None if there are no groups)

        Parameters
        ----------
        model : Model
            Model to get last group in

        Returns
        -------
        Group
            Group object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Last", model)

    def Total(model):
        """
        Returns the total number of groups in a model

        Parameters
        ----------
        model : Model
            Model to get group in

        Returns
        -------
        integer
            The number of groups
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Total", model)



# Instance methods
    def AddFlagged(self, flag):
        """
        Adds flagged items to the contents of the group

        Parameters
        ----------
        flag : Flag
            Flag (see AllocateFlag) set on items to add to the group

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "AddFlagged", flag)

    def Empty(self):
        """
        Empties the group (removes everything from the group)

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Empty")

    def FlagContents(self, flag):
        """
        Flags the contents of the group

        Parameters
        ----------
        flag : Flag
            Flag (see AllocateFlag) to set for the group contents

        Returns
        -------
        None
            No return value
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "FlagContents", flag)

    def Next(self):
        """
        Returns the next group in the model (or None if there is not one)

        Returns
        -------
        Group
            Group object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Next")

    def Previous(self):
        """
        Returns the previous group in the model (or None if there is not one)

        Returns
        -------
        Group
            Group object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Previous")




import Oasys.gRPC


# Metaclass for static properties and constants
class TypeType(type):
    _consts = {'BEAM', 'BOLT', 'BWLD', 'CONTACT', 'CONX', 'CWLD', 'DES', 'ELEMENT', 'GROUP', 'GWLD', 'HSWA', 'HWLD', 'JOINT', 'MASS', 'MATERIAL', 'MIG', 'MODEL', 'NODE', 'NRB', 'PART', 'PRETENSIONER', 'RBOLT', 'RETRACTOR', 'RIGIDWALL', 'SBENT', 'SEATBELT', 'SECTION', 'SEGMENT', 'SET_BEAM', 'SET_DISCRETE', 'SET_NODE', 'SET_PART', 'SET_SHELL', 'SET_SOLID', 'SET_TSHELL', 'SHELL', 'SLIPRING', 'SOLID', 'SPC', 'SPH', 'SPRING', 'TSHELL', 'WINDOW', 'XSEC'}

    def __getattr__(cls, name):
        if name in TypeType._consts:
            return Oasys.D3PLOT._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Type(Oasys.gRPC.OasysItem, metaclass=TypeType):


    def __del__(self):
        if not Oasys.D3PLOT._connection:
            return

        Oasys.D3PLOT._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value



import Oasys.gRPC


# Metaclass for static properties and constants
class ColourType(type):
    _consts = {'BLACK', 'BLUE', 'CYAN', 'DARK_ORANGE', 'DEFAULT', 'GREEN', 'GREEN_CYAN', 'GREY', 'LIGHT_BLUE', 'MAGENTA', 'MEDIUM_BLUE', 'ORANGE', 'RED', 'RED_MAGENTA', 'WHITE', 'YELLOW', 'YELLOW_GREEN'}

    def __getattr__(cls, name):
        if name in ColourType._consts:
            return Oasys.D3PLOT._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Colour(Oasys.gRPC.OasysItem, metaclass=ColourType):


    def __del__(self):
        if not Oasys.D3PLOT._connection:
            return

        Oasys.D3PLOT._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value


# Static methods
    def GetFromName(name):
        """
        Returns the colour for a given core or user colour name

        Parameters
        ----------
        name : string
            The name of the colour, for example red or user_green or green/cyan

        Returns
        -------
        int
            colour value (integer)
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFromName", name)

    def RGB(red, green, blue):
        """
        Creates a colour from red, green and blue components

        Parameters
        ----------
        red : integer
            red component of colour (0-255)
        green : integer
            green component of colour (0-255)
        blue : integer
            blue component of colour (0-255)

        Returns
        -------
        int
            colour value (integer)
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "RGB", red, green, blue)




import Oasys.gRPC


# Metaclass for static properties and constants
class IncludeType(type):
    _consts = {'NATIVE', 'UNIX', 'WINDOWS'}

    def __getattr__(cls, name):
        if name in IncludeType._consts:
            return Oasys.D3PLOT._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Include(Oasys.gRPC.OasysItem, metaclass=IncludeType):
    _props = {'label', 'name', 'parent'}


    def __del__(self):
        if not Oasys.D3PLOT._connection:
            return

        Oasys.D3PLOT._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
# If one of the properties we define then get it
        if name in Include._props:
            return Oasys.D3PLOT._connection.instanceGetter(self.__class__.__name__, self._handle, name)

        raise AttributeError


    def __setattr__(self, name, value):
# If one of the properties we define then set it
        if name in Include._props:
            Oasys.D3PLOT._connection.instanceSetter(self.__class__.__name__, self._handle, name, value)
            return

# Set the property locally
        self.__dict__[name] = value


# Static methods
    def First(model):
        """
        Returns the first include file in the model (or None if there are no include files in the model)

        Parameters
        ----------
        model : Model
            Model to get first include file in

        Returns
        -------
        Include
            Include object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "First", model)

    def GetFromID(model, number):
        """
        Returns the include file in the model with number (or None if it does not exist)

        Parameters
        ----------
        model : Model
            Model to get include file in
        number : integer
            The number for the include file in the model. Note that include file numbers start at 1. 0 is the main file

        Returns
        -------
        Include
            Include object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "GetFromID", model, number)

    def Last(model):
        """
        Returns the last include file in the model (or None if there are no include files in the model)

        Parameters
        ----------
        model : Model
            Model to get last include file in

        Returns
        -------
        Include
            Include object
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Last", model)

    def Total(model):
        """
        Returns the total number of include files in the model

        Parameters
        ----------
        model : Model
            Model to get total in

        Returns
        -------
        int
            Number of includes
        """
        return Oasys.D3PLOT._connection.classMethod(__class__.__name__, "Total", model)



# Instance methods
    def Next(self):
        """
        Returns the next include file in the model (or None if there is not one)

        Returns
        -------
        Include
            Include object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Next")

    def Previous(self):
        """
        Returns the previous include file in the model (or None if there is not one)

        Returns
        -------
        Include
            Include object
        """
        return Oasys.D3PLOT._connection.instanceMethod(self.__class__.__name__, self._handle, "Previous")

